@@ -40,7 +40,7 @@ class MDCluster;
 class Context;
 class bloom_filter;
 
-class ObjectOperation;
+struct ObjectOperation;
 
 ostream& operator<<(ostream& out, class CDir& dir);
 class CDir : public MDSCacheObject {
@@ -140,7 +140,7 @@ public:
   static const int REP_LIST =     2;
 
 
-  static const int NONCE_EXPORT  = 1;
+  static const unsigned EXPORT_NONCE  = 1;
 
 
   // -- wait masks --
@@ -286,6 +286,7 @@ protected:
  public:
   CDir(CInode *in, frag_t fg, MDCache *mdcache, bool auth);
   ~CDir() {
+    remove_bloom();
     g_num_dir--;
     g_num_dirs++;
   }
@@ -357,6 +358,7 @@ private:
   void remove_null_dentries();
   void purge_stale_snap_data(const set<snapid_t>& snaps);
 public:
+  void touch_dentries_bottom();
   bool try_trim_snap_dentry(CDentry *dn, const set<snapid_t>& snaps);
 
 
@@ -413,7 +415,7 @@ private:
   // for giving to clients
   void get_dist_spec(set<int>& ls, int auth) {
     if (is_rep()) {
-      for (map<int,int>::iterator p = replicas_begin();
+      for (map<int,unsigned>::iterator p = replicas_begin();
 	   p != replicas_end(); 
 	   ++p)
 	ls.insert(p->first);
