@@ -162,6 +162,48 @@ private:
 	   << " current snap is " << context.current_snap << std::endl;
       return new CopyFromOp(m_op, &context, oid, oid2, m_stats);
 
+    case TEST_OP_HIT_SET_LIST:
+      {
+	uint32_t hash = rjhash32(rand());
+	cout << "hit_set_list " << hash << std::endl;
+	return new HitSetListOp(m_op, &context, hash, m_stats);
+      }
+
+    case TEST_OP_UNDIRTY:
+      {
+	oid = *(rand_choose(context.oid_not_in_use));
+	cout << "undirty oid " << oid << std::endl;
+	return new UndirtyOp(m_op, &context, oid, m_stats);
+      }
+
+    case TEST_OP_IS_DIRTY:
+      {
+	oid = *(rand_choose(context.oid_not_in_use));
+	cout << "is_dirty oid " << oid << std::endl;
+	return new IsDirtyOp(m_op, &context, oid, m_stats);
+      }
+
+    case TEST_OP_CACHE_FLUSH:
+      {
+	oid = *(rand_choose(context.oid_not_in_use));
+	cout << "cache_flush oid " << oid << std::endl;
+	return new CacheFlushOp(m_op, &context, oid, m_stats, true);
+      }
+
+    case TEST_OP_CACHE_TRY_FLUSH:
+      {
+	oid = *(rand_choose(context.oid_not_in_use));
+	cout << "cache_try_flush oid " << oid << std::endl;
+	return new CacheFlushOp(m_op, &context, oid, m_stats, false);
+      }
+
+    case TEST_OP_CACHE_EVICT:
+      {
+	oid = *(rand_choose(context.oid_not_in_use));
+	cout << "cache_evict oid " << oid << std::endl;
+	return new CacheEvictOp(m_op, &context, oid, m_stats);
+      }
+
     default:
       cerr << "Invalid op type " << type << std::endl;
       assert(0);
@@ -203,6 +245,12 @@ int main(int argc, char **argv)
     { TEST_OP_TMAPPUT, "tmapput" },
     { TEST_OP_WATCH, "watch" },
     { TEST_OP_COPY_FROM, "copy_from" },
+    { TEST_OP_HIT_SET_LIST, "hit_set_list" },
+    { TEST_OP_IS_DIRTY, "is_dirty" },
+    { TEST_OP_UNDIRTY, "undirty" },
+    { TEST_OP_CACHE_FLUSH, "cache_flush" },
+    { TEST_OP_CACHE_TRY_FLUSH, "cache_try_flush" },
+    { TEST_OP_CACHE_EVICT, "cache_evict" },
     { TEST_OP_READ /* grr */, NULL },
   };
 
